@@ -197,28 +197,6 @@ public class NSDomainHandler extends EPPDomainHandler {
 	 * Constructs an instance of NSDomainHandler
 	 */
 	public NSDomainHandler() {
-
-		// Add NameStore domain extensions and poll message mappings
-		try {
-			cat.info("NSDomainHandler(): Registering EPPLowBalancePollMapFactory");
-			EPPFactory.getInstance().addMapFactory(
-					EPPLowBalancePollMapFactory.class.getName());
-			cat.info("NSDomainHandler(): Registering EPPRgpPollMapFactory");
-			EPPFactory.getInstance().addMapFactory(
-					EPPRgpPollMapFactory.class.getName());
-			cat.info("NSDomainHandler(): Registering EPPRgpExtFactory");
-			EPPFactory.getInstance().addExtFactory(
-					EPPRgpExtFactory.class.getName());
-
-			cat.info("NSDomainHandler(): Registering EPPSyncExtFactory");
-			EPPFactory.getInstance().addExtFactory(
-					EPPSyncExtFactory.class.getName());
-
-		}
-		catch (EPPCodecException ex) {
-			cat.error("NSDomainHandler(): Couldn't load the Map Factory", ex);
-			System.exit(1);
-		}
 	}
 
 	/**
