@@ -324,7 +324,24 @@ public class EPPSystemSessionPool  {
 						": " + ex);
 				throw new EPPSessionPoolException("Exception creating instance of class " + theValue + 
 						": " + ex);
-			}			
+			}	
+			
+			// clientTransIdGenerator
+			theValue = this.getProperty("clientTransIdGenerator");
+			log.info("init(): client trans id generator = " + theValue);
+			
+			if (theValue != null) {
+				try {
+					this.factory.setClientTransIdGenerator((EPPClientTransIdGenerator) Class.forName(theValue).newInstance());				
+				}
+				 catch (Exception ex) {
+					log.error("EPPSystemSessionPool.init(): Exception creating instance of class " + theValue + 
+							": " + ex);
+					throw new EPPSessionPoolException("Exception creating instance of class " + theValue + 
+							": " + ex);
+				}					
+			}
+			
 			 
 			// serverName
 			this.serverName = this.getProperty("serverName");
